#!/usr/bin/perl -w
#
# This Perl script creates an tab-delimited text file(s) based on the
# information in the Post.Office account database. The tab-delimited
# file can be used to create accounts on your new CommuniGate Pro
# server, using its Account Import feature. 
# <http://www.stalker.com/CommuniGatePro/Accounts.html#Loader>
#
# The script should be used on the same computer where the Post.Office
# software is installed (the Post.Office is not required to be running) 
# See <http://www.stalker.com/CGMigration/CGPostOffice.html>
#
# Instructions:
# 1. Save the script into the directory where Post.Office "listacct"
#    utility is located (e.g. C:\win32app\Post.Office\cmdutils\ or
#    /usr/local/post.office/cmdutils/).
# 2. Launch the script (e.g. "perl PostOfficeToCGP.pl")
# 3. For each of your PostOffice domains the "import-domain.name" file will
#    appear. Use those files with CommuniGate Pro WebAdmin Account Import.
#     
# Mail your comments and suggestions to <support@stalker.com>
#

if($^O eq 'MSWin32') {

  $execList = "listacct.exe";
  $execAccnt = "getacct.exe";

  $inputFileName = "listaccnt.tmp";
  $tempFileName = "getaccnt.tmp";
  $outputPrefix = "import-";
  $outputPrefix2 = ".txt";

} else {

  $execList = "./listacct";
  $execAccnt = "./getacct";

  $inputFileName = "/tmp/listaccnt.tmp";
  $tempFileName = "/tmp/getaccnt.tmp";
  $outputPrefix = "import-";
}

sub PutAddresses($$$$);
sub PutRules($$$$$);

my %domainList;

print "Starting\n";

my $execStr="$execList -i Account-ID,Name,SMTP-Address,POP-Address,Password,Mailbox-Quota,Local-Delivery,Handler-Delivery,Forward-Delivery >$inputFileName";

print "Launching $execList\n";
system($execStr)==0 or die "Can't execute $execStr\n" ; 
 
 
die "Can't find $inputFileName\n" unless -e $inputFileName;

print "Importing accounts...\n";
open INFILE, "< $inputFileName" or die "Can't open $inputFileName: $!\n";

while(<INFILE>) {
  chomp;
  my ($id,$realName,$addresses,$popAddr,$password,$quota,$lDelivery,$hDelivery,$fDelivery) = split /;/;
  if($lDelivery ne 'Mailbox' && $password eq '') {
    print "Not an importable account:\n$_\n\n";
  } else {
    $addresses =~ tr/A-Z/a-z/;
    $addresses =~ /\@([\w\.\[\]]+)/g;
    my $domain=$1;
    if(defined $domain && $domain ne '') {
	    unless(exists($domainList{$domain})) {
	      print "Creating new file for $domain\n";
	      open $domain,"> $outputPrefix$domain$outputPrefix2" or die "Can't open $outputPrefix$domain$outputPrefix2: $!\n";
	      print $domain "Name\tRealName\tAliases\tUnixPassword\tStorage\tRules\n";
	      $domainList{$domain} = 0;
	    }
	    PutAddresses($domain,$realName,$addresses,$popAddr);
	    print $domain "{NS-MTA-MD5}$password\t";
	    
	    if($quota) { print $domain $quota."K";}
	    print $domain "\t";
	    
	    PutRules($domain,$id,$lDelivery,$hDelivery,$fDelivery);   
	    
	    print $domain "\n";
	    $domainList{$domain}++;
    }
  }
}

close INFILE;
unlink $inputFileName;
unlink $tempFileName;

my $total=0;
print "----------------------------------------\n"; 
foreach(keys %domainList) {
  close $_ or print "Can't close for $_: $!\n";
  print "$_ : $domainList{$_}\n";
  $total+=$domainList{$_};
}
print "----------------------------------------\nTotal: $total accounts\n"; 
exit;

sub PutAddresses($$$$) {
  my ($domain,$realName,$addresses,$popAddr) = @_;
  my %addrList;
  my @tmpList=split /,/,$addresses;

  $tmpList[0] =~/^(.+)\@/;

  my $accountName=$1;
  $addrList{$accountName}=$accountName;
  shift @tmpList;

  print $domain "$accountName\t$realName\t";

  push @tmpList,$popAddr;

  foreach(@tmpList) {
    my $xAlias=$_;
    if($xAlias =~ /(.+)\@(.+)/) {
      if($2 ne $domain) {
        # print "wrong domain\n";
        next;
      }
      $xAlias=$1;
    }
    if(exists($addrList{$xAlias})) {
      # print "duplicate address: $_\n";
    } else { 
      $addrList{$xAlias}=$xAlias;
    }    
  }

  delete($addrList{$accountName});  
  print $domain join(",",keys %addrList)."\t"; 
}

sub PutRules($$$$) {
  my($domain,$id,$lDelivery,$hDelivery,$fDelivery) = @_;
  print $domain "(";
  if($fDelivery) {
    my @tmpList=split /,/,$fDelivery;
    print $domain "(1,\"#Redirect\",((\"Human Generated\", \"---\")), ((\"Redirect to\",\"";
    foreach(@tmpList) {
      s/SMTP|<|>| //g;
    }
    print $domain join(",",@tmpList);
    print $domain "\")";
    print $domain ",(Discard, \"---\")" if($lDelivery ne 'Mailbox');
    print $domain "))";
  }
  if($hDelivery eq 'AutoReply-Handler') {
    my $exec2Str="$execAccnt $id - >$tempFileName";
    if(system($exec2Str)==0) {
      if(open(INP, "< $tempFileName")!=0) {
				my $replyStr="";
				readReply:while(<INP>) {
				  chomp;
				  if(/^AutoReply-Info: \[(.*)\]/) {
				    $replyStr.=$1."\n";
				    while(<INP>) {
				      if(/^\s*\[(.*)\]/) {
				        $replyStr.=$1."\n";
				      } else {		        
				        last readReply;
				      }
				    }
				  }
				}				
				close(INP);
				$replyStr =~ s/\\/\\\\/g;
				$replyStr =~ s/\"/\\\"/g;
				$replyStr =~ s/\n/\\e/g;
        print $domain "," if($fDelivery);
        print $domain "(1,\"#Vacation\",((\"Human Generated\", \"---\")),((\"Reply with\", \"$replyStr\")))";
      }    
    } else {
      print "unable to execute: $exec2Str\n";
    } 
  } 
  print $domain ")"; 
}
